""" file:   __init__.py (pysiss.vocabulary)
    author: Jess Robertson
            Mineral Resources Flagship, CSIRO
    date:   something

    description: init file
"""

import gsml
import gml
import erml
import namespaces

__all__ = [gsml, gml, erml, namespaces]
