""" file:   __init__.py (pysiss.borehole.siss)
    author: David Benn
            CSIRO IM&T Scientific Computing Data Processing Services
    email:  David.Benn@csiro.au
    date:   Tuesday May 20, 2014

    description: Initialisation of the pysiss.borehole.siss module.
"""

import borehole_generator